inspected_players = (inspected_players == nil) and {}

function inspect_player(uid)
    -- print_chat(UnitGUID(uid))
    local inst, _ = IsInInstance()
    if inst then
        return
    end
    if not UnitIsPlayer(uid) then
        return
    end
    local guid = UnitGUID(uid)
    if guid == nil then
        return
    end
    if inspected_players[guid] == nil then
        inspected_players[guid] = {
            h = 0,
            a = 0,
            d = "",
            t = 0
        }
    end
    local name, realm = UnitName(uid)
    realm = (realm == nil) and GetRealmName() or realm
    local _, faction = UnitFactionGroup(uid)
    local level = UnitLevel(uid)
    local gender = (UnitSex(uid) == 2) and "男" or "女"
    local race, _ = UnitRace(uid)
    local class, _, _ = UnitClass(uid)
    local description = string.format("%s,%d级%s%s%s,%s%s", name, level, gender, race, class, realm, faction)
    local guild, grank, gindex = GetGuildInfo(uid)
    if guild ~= nil then
        description = string.format("%s,%s%d阶%s", description, guild, gindex, grank)
    end
    -- 不要把成就点清0了
    inspected_players[guid].h = UnitHonorLevel(uid)
    inspected_players[guid].d = description
    inspected_players[guid].t = time()
    if uid == "target" then
        SetAchievementComparisonUnit(uid)
    end
end

function setup_player_inspector()
    register_event(
        "INSPECT_ACHIEVEMENT_READY",
        function(guid)
            local ap = GetComparisonAchievementPoints()
            if inspected_players[guid].a ~= nil then
                if inspected_players[guid].a <= ap then
                    inspected_players[guid].a = ap
                    print_chat(
                        "|cFFFF6633" ..
                            inspected_players[guid].h ..
                                ", |cFF00CC00" ..
                                    inspected_players[guid].a .. ", |cFF99CCFF" .. inspected_players[guid].d
                    )
                else
                    inspected_players[guid].ah = true
                    print_chat(
                        "|cFFFF6633" ..
                            inspected_players[guid].h ..
                                ", |cFF00CC00" ..
                                    inspected_players[guid].a .. "（已隐藏）, |cFF99CCFF" .. inspected_players[guid].d
                    )
                end
            end
            ClearAchievementComparisonUnit()
        end
    )
    register_event(
        "PLAYER_TARGET_CHANGED",
        function()
            inspect_player("target")
        end
    )
    register_event(
        "UPDATE_MOUSEOVER_UNIT",
        function()
            inspect_player("mouseover")
        end
    )
    local btn =
        create_button(
        "玩家检查",
        function()
            local t = {}
            for k, v in pairs(inspected_players) do
                if v.a ~= 0 then
                    table.insert(t, {v.h, v.a, v.d})
                end
            end
            table.sort(
                t,
                function(a, b)
                    if a[1] == b[1] then
                        return a[2] < b[2]
                    else
                        return a[1] < b[1]
                    end
                end
            )
            ConsoleExec("clear")
            print(".")
            print("======== HonorLevel, AchievementPoints, Description ========")
            local pv = {0, 0, ""}
            for _, v in ipairs(t) do
                if abs(v[1] - pv[1]) <= 0 and abs(v[2] - pv[2]) <= 50 then
                    pv.show = true
                    v.show = true
                end
                pv = v
            end
            for _, v in ipairs(t) do
                if v.show then
                    print(v[1], v[2], v[3])
                end
            end
        end
    )
    btn:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 0, 0)
end
