local N, T = ...

function T.Setup_HUD()
    local frame = CreateFrame("Frame", nil, UIParent)
    frame:SetFrameStrata("BACKGROUND")
    frame:SetSize(512, 512)
    frame:SetAlpha(0.6)
    frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)

    local t = frame:CreateTexture(nil, "BACKGROUND")
    t:SetTexture("Interface/AddOns/" .. N .. "/res/cross1.tga")
    t:SetAlpha(0.4)
    t:SetAllPoints()

    local function create_label(...)
        local l = frame:CreateFontString(nil, "BACKGROUND", nil)
        l:SetFont("Fonts\\ARHei.TTF", 14, "OUTLINE")
        l:SetPoint(...)
        return l
    end

    local combat_label = create_label("TOPRIGHT", frame, "RIGHT", 0, -5)
    combat_label:SetText("COMBAT")
    combat_label:SetShown(InCombatLockdown())
    T.Register_Event(
        "PLAYER_REGEN_DISABLED",
        function()
            combat_label:SetShown(true)
        end
    )
    T.Register_Event(
        "PLAYER_REGEN_ENABLED",
        function()
            combat_label:SetShown(false)
        end
    )

    local afk_label = create_label("TOPRIGHT", combat_label, "BOTTOMRIGHT", 0, -5)
    afk_label:SetText("AFK")
    afk_label:SetShown(UnitIsAFK("player"))
    T.Register_Event(
        "PLAYER_FLAGS_CHANGED",
        function(event, ...)
            local target = ...
            if target == "player" then
                afk_label:SetShown(UnitIsAFK("player"))
            end
        end
    )

    local pos_label = create_label("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0)
    local spd_label = create_label("BOTTOMRIGHT", pos_label, "TOPRIGHT", 0, 5)
    local moving_ticker = nil
    local function update_pos_spd_label()
        if not moving_ticker then
            -- print("moving_ticker start")
            moving_ticker =
                C_Timer.NewTicker(
                0.5,
                function()
                    -- 位置坐标
                    pos_label:SetText("N/A")
                    local mapid = C_Map.GetBestMapForUnit("player") -- 进入副本一瞬间会返回nil
                    if mapid then
                        local pos = C_Map.GetPlayerMapPosition(mapid, "player")
                        if pos then
                            local x, y = pos:GetXY()
                            pos_label:SetText(string.format("%.1f %.1f", x * 100, y * 100))
                        end
                    end
                    -- 移动速度
                    local c, _, _, _ = GetUnitSpeed("player")
                    spd_label:SetText(string.format("%d", c * 100 / BASE_MOVEMENT_SPEED))
                    if (not IsPlayerMoving()) and (not UnitOnTaxi("player")) and moving_ticker then
                        -- print("moving_ticker stop")
                        moving_ticker:Cancel()
                        moving_ticker = nil
                    end
                end
            )
        end
    end
    update_pos_spd_label()
    T.Register_Event("PLAYER_ENTERING_WORLD", update_pos_spd_label)
    T.Register_Event("PLAYER_STARTED_MOVING", update_pos_spd_label)
    T.Register_Event("TAXIMAP_CLOSED", update_pos_spd_label)

    local instdiff_label = create_label("BOTTOMRIGHT", frame, "RIGHT", 0, 5)
    local function update_instdiff_label()
        local _, _, _, dname = GetInstanceInfo()
        instdiff_label:SetText(dname)
    end
    update_instdiff_label()
    hooksecurefunc("MiniMapInstanceDifficulty_Update", update_instdiff_label)
    T.Register_Event("ZONE_CHANGED", update_instdiff_label)
    T.Register_Event("ZONE_CHANGED_INDOORS", update_instdiff_label)
    T.Register_Event("ZONE_CHANGED_NEW_AREA", update_instdiff_label)
end
