local N, T = ...

function T.Setup_UI()
    if T.IsAuthor then
        -- 是否为试玩状态
        -- IsRestrictedAccount()、IsTrialAccount()以及GameLimitedMode_IsActive()这两个函数待测试
        local is_trial = GameLimitedMode_IsActive()

        -- CVars设置
        for k, v in pairs(
            {
                scriptErrors = "0", -- 显示Lua错误
                ffxDeath = "0",
                ffxGlow = "0",
                ffxNether = "0",
                weatherDensity = "0",
                overrideArchive = "0", -- 反和谐
                rawMouseEnable = "0", -- 鼠标偶然乱晃问题 2020.07.23更新后如果为1那么右键不按下也会旋转屏幕
                -- 控制
                autoLootDefault = "0", -- 自动拾取
                -- 名字
                nameplateShowAll = "1", -- 显示所有姓名板
                nameplateShowEnemyMinus = "0", -- 不显示杂兵姓名板
                nameplateShowSelf = "1", -- 自己的姓名板
                -- 团队界面配置
                useCompactPartyFrames = "1", -- 团队风格小队框体
                -- 地图
                showTamers = is_trial and "0" or "1", -- 地图显示宠物对战训练师
                questLogOpen = "0", -- 地图右侧显示任务
                mapFade = "0", -- 移动时地图透明
                -- 任务
                autoQuestWatch = "1", -- 接任务时自动追踪
                autoQuestProgress = "1", -- 任务更新时自动追踪
                -- 姓名板
                nameplateMaxDistance = "60", -- 姓名板距离
                nameplateOtherTopInset = 0.08, -- 敌方姓名板与屏幕顶端的最少距离（默认0.08）
                nameplateOtherBottomInset = 0.1, -- 敌方姓名板与屏幕底端的最少距离（默认0.1）
                nameplateSelfTopInset = 0.5, -- 自己姓名板与屏幕顶端的最少距离（默认0.5）
                nameplateSelfBottomInset = 0.2, -- 自己姓名板与屏幕底端的最少距离（默认0.2）
                nameplateMotion = 0, -- 姓名板排列方式
                -- 其它
                cameraDistanceMaxZoomFactor = "1.2", -- 镜头最远距离（15的倍数）
                screenshotFormat = "png",
                screenshotQuality = "10",
                chatBubbles = "1",
                chatStyle = "im",
                whisperMode = "popout",
                statusText = "1",
                statusTextDisplay = "BOTH", -- 姓名板显示状态文字
                showToastWindow = "0" -- 战网信息显示在浮窗中
            }
        ) do
            C_CVar.SetCVar(k, v)
        end

        T.Register_Event(
            "PLAYER_ENTERING_WORLD",
            function()
                local realm_name = GetRealmName()
                local in_instance, instance_type = IsInInstance()
                local unit_name_cvars = {
                    "UnitNameFriendlyGuardianName",
                    "UnitNameFriendlyMinionName",
                    "UnitNameFriendlyPetName",
                    "UnitNameFriendlyPlayerName",
                    "UnitNameFriendlyTotemName",
                    "UnitNameEnemyGuardianName",
                    "UnitNameEnemyMinionName",
                    "UnitNameEnemyPetName",
                    "UnitNameEnemyPlayerName",
                    "UnitNameEnemyTotemName"
                }
                local v
                if realm_name == "金色平原" or (in_instance and (instance_type == "pvp" or instance_type == "arena")) then
                    v = 1
                else
                    v = 0
                end
                for i, k in ipairs(unit_name_cvars) do
                    C_CVar.SetCVar(k, v)
                end
            end
        )

        -- 小地图追踪设置
        local trackings_enabled = {
            -- ["追踪宠物"] = not is_trial,
            ["飞行管理员"] = true,
            ["其他任务"] = true,
            ["关注点"] = true,
            ["目标"] = true,
            ["焦点目标"] = true,
            ["追踪任务目标数据"] = true,
            ["追踪挖掘场"] = not is_trial,
            -- 镇民
            ["旅店老板"] = true,
            ["邮箱"] = true,
            ["兽栏管理员"] = true,
            -- 专业技能
            ["寻找矿物"] = true,
            ["寻找草药"] = true
            -- ["寻找渔点"] = true,
            -- 猎人，德鲁伊
            -- ["追踪亡灵"] = true,
            -- ["追踪人型生物"] = true,
            -- ["追踪元素生物"] = true,
            -- ["追踪巨人"] = true,
            -- ["追踪恶魔"] = true,
            -- ["追踪机械"] = true,
            -- ["追踪野兽"] = true,
            -- ["追踪隐藏生物"] = true,
            -- ["追踪龙类"] = true
        }
        for i = 1, GetNumTrackingTypes() do
            local name, texture, active, category = GetTrackingInfo(i)
            SetTracking(i, trackings_enabled[name])
        end

        -- 隐藏账户成就，含义貌似反了
        ShowAccountAchievements(true)

        -- 反向整理背包
        SetSortBagsRightToLeft(true)
    end

    -- 动作条
    for _, f in ipairs(
        {
            MultiBarBottomLeft,
            MultiBarBottomRight,
            MultiBarLeft,
            MultiBarRight,
            StanceBarFrame,
            PetActionBarFrame
        }
    ) do
        f:SetAlpha(0.5)
    end
    MainMenuBarArtFrame.LeftEndCap:Hide()
    MainMenuBarArtFrame.RightEndCap:Hide()
    MainMenuBarArtFrameBackground:Hide()

    -- 小地图
    MinimapCluster:SetScale(1.2)
    MinimapBorderTop:Hide()
    MinimapBorder:Hide()
    Minimap:SetMaskTexture("Interface/ChatFrame/ChatFrameBackground")
    Minimap:SetPlayerTexture("Interface/AddOns/" .. N .. "/res/minimap_arrow_3.tga")

    -- 屏幕上方错误显示
    UIErrorsFrame:SetAlpha(0.5)

    -- 屏幕上的按钮
    -- ExtraActionButton1:SetScale(0.7)
    ExtraActionButton1:SetAlpha(0.5)
    ExtraActionButton1.style:Hide()
    -- ZoneAbilityFrame:SetScale(0.7)
    ZoneAbilityFrame:SetAlpha(0.5)
    ZoneAbilityFrame.Style:Hide()
    PlayerPowerBarAlt:SetAlpha(0.5) -- 某些任务出现的
    ObjectiveTrackerFrame:SetAlpha(0.5) -- 任务列表

    -- 复活按钮，GhostFrame是“返回墓地”按钮
    local btn_resurrect = CreateFrame("Button", nil, GhostFrame, "UIPanelButtonTemplate")
    btn_resurrect:SetText("复活")
    btn_resurrect:SetSize(btn_resurrect:GetTextWidth() + 12, btn_resurrect:GetTextHeight() + 8)
    btn_resurrect:SetPoint("BOTTOM", GhostFrame, "TOP", 0, 0)
    btn_resurrect:SetParent(GhostFrame)
    btn_resurrect:SetScript(
        "OnClick",
        function(self, button, down)
            RetrieveCorpse()
        end
    )
    btn_resurrect:Disable()
    btn_resurrect:SetScript(
        "OnEvent",
        function(self, event, ...)
            -- print(event)
            if event == "CORPSE_IN_RANGE" then
                self:Enable()
            elseif event == "CORPSE_OUT_OF_RANGE" then
                self:Disable()
            end
        end
    )
    btn_resurrect:RegisterEvent("CORPSE_IN_RANGE")
    btn_resurrect:RegisterEvent("CORPSE_OUT_OF_RANGE")
    T.Hook_StaticPopup( -- 隐藏默认复活对话框
        "RECOVER_CORPSE",
        function()
            StaticPopup_Hide("RECOVER_CORPSE")
        end
    )

    -- 考古进度条
    T.On_AddOn_Loaded(
        "Blizzard_ArchaeologyUI",
        function()
            hooksecurefunc(
                ArcheologyDigsiteProgressBar.AnimIn,
                "Play",
                function()
                    ArcheologyDigsiteProgressBar.AnimIn:Stop()
                    ArcheologyDigsiteProgressBar:SetAlpha(0.5)
                    ArcheologyDigsiteProgressBar.Shadow:Hide()
                end
            )
        end
    )

    -- 小地图要塞按钮强制显示
    hooksecurefunc(
        GarrisonLandingPageMinimapButton,
        "Hide",
        function()
            GarrisonLandingPageMinimapButton:Show()
        end
    )
    GarrisonLandingPageMinimapButton:Show()

    -- 设置地图标记
    local obj = {
        name = "SET_USER_WAYPOINT",
        commands = {"/wp"}
    }
    obj.callback = function(sx, sy)
        local x = tonumber(sx)
        local y = tonumber(sy)
        if x == nil or y == nil then
            print("Usage:", unpack(obj.commands), "x y")
            return
        end
        local map_id = C_Map.GetBestMapForUnit("player")
        if C_Map.CanSetUserWaypointOnMap(map_id) then
            local point = UiMapPoint.CreateFromCoordinates(map_id, x / 100, y / 100, 0)
            C_Map.SetUserWaypoint(point)
            C_SuperTrack.SetSuperTrackedUserWaypoint(true)
        else
            print("Cannot set waypoints on this map")
        end
    end
    T.Add_Slash_Command(obj)

    -- 德鲁伊变形乌鸦形态的声音屏蔽
    -- https://wow.tools/files/#search=ravenmount%2Ctype%3Aogg&page=1&sort=0&desc=asc
    for i = 1570532, 1570536 do
        MuteSoundFile(i)
    end
    for i = 1570545, 1570546 do
        MuteSoundFile(i)
    end
    MuteSoundFile(1570541)

    -- 弹出对话框贴顶显示
    -- BlizzardInterfaceCode\Interface\FrameXML\StaticPopup.lua
    hooksecurefunc(
        "StaticPopup_SetUpAnchor",
        function(dialog, idx)
            local lastFrame = StaticPopup_DisplayedFrames[idx - 1]
            if not lastFrame then
                dialog:SetPoint("TOP", UIParent, "TOP", 0, dialog.topOffset or 0)
            end
        end
    )
    -- 去掉弹出对话框的边框
    hooksecurefunc(
        "StaticPopup_OnShow",
        function(self)
            for i, v in ipairs(
                {
                    "TopEdge",
                    "BottomEdge",
                    "LeftEdge",
                    "RightEdge",
                    "TopLeftCorner",
                    "TopRightCorner",
                    "BottomLeftCorner",
                    "BottomRightCorner"
                }
            ) do
                self.Border[v]:Hide()
            end
        end
    )

    -- 打印插件状态
    T.Add_Slash_Command(
        {
            name = "PRINT_ADDON_STATES",
            commands = {"/pas"},
            callback = function(...)
                local states = {}
                for i = 1, GetNumAddOns() do
                    if IsAddOnLoaded(i) then
                        table.insert(states, {({GetAddOnInfo(i)})[1], GetAddOnMemoryUsage(i)})
                    end
                end
                table.sort(
                    states,
                    function(a, b)
                        return a[2] > b[2]
                    end
                )
                for i, v in ipairs(states) do
                    print(string.format("%s %.2f", v[1], v[2]))
                end
            end
        }
    )

    GameTooltip:HookScript(
        "OnTooltipSetSpell",
        function(self)
            local _, spell_id = self:GetSpell()
            if not spell_id then
                return
            end
            self:AddLine("spell_id = " .. spell_id)
            self:AddLine("GetSpellBaseCooldown() = " .. T.ToString(GetSpellBaseCooldown(spell_id)))
            self:AddLine("GetSpellCooldown() = " .. T.ToString(GetSpellCooldown(spell_id)))
        end
    )

    -- 聊天框取消上下左右键必须按alt生效的设定
    hooksecurefunc(
        "ChatEdit_ActivateChat",
        function(self)
            self:SetAltArrowKeyMode(false)
        end
    )
    -- for i = 1, NUM_CHAT_WINDOWS do
    --     local editbox = _G["ChatFrame" .. i .. "EditBox"]
    --     editbox:HookScript(
    --         "OnArrowPressed",
    --         function(self, key)
    --             ConsoleAddMessage(key)
    --             AutoCompleteEditBox_OnArrowPressed(self, key)
    --         end
    --     )
    -- end
end
